<?php
/**
 * fmminimize.php
 *
 * php version 5
 *
 * @category  Joomla
 * @package   Joomla
 * @author    Folcomedia <contact@folcomedia.fr>
 * @copyright 2014 Folcomedia
 * @license   http://www.gnu.org/licenses/gpl.html
 * @link      https://www.folcomedia.fr
 */

defined('_JEXEC') or die;

class plgSystemFMMinimize extends JPlugin {

    /**
     * Intercepte la requete HTTP pour la traiter si elle concerne une miniature
     */
    function onAfterInitialise()
    {

        $app = JFactory::getApplication();

        // Récupérer des dimensions d'images (auto réduction)
        /*if ($app->input->get('fmmGetDimensions')) {
            $dimensions = array();
            foreach ($app->input->get('fmmData', array(), 'ARRAY') as $img) {
                $path = str_replace(JURI::root(), JPATH_BASE.'/', $img);
                if (is_file($path)) {
                    $size = getimagesize($path);
                    array_push($dimensions, $size[0].'x'.$size[1]);
                } else {
                    array_push($dimensions, '0x0');
                }
            }
            $return = new stdClass();
            $return->dimensions = $dimensions;
            echo json_encode($return);
            exit();
        }*/

        // Ecriture de la config si nécessaire
        $this->writeConfig();

        // SEF ?
        if (!$app->getCfg('sef_rewrite') && !(JRequest::get('post'))) {
            $lang = JFactory::getLanguage();
            $lang->load('plg_system_fmminimize', JPATH_ADMINISTRATOR);
            JError::raiseWarning(101, JText::_('PLG_SYSTEM_FMMINIMIZE_SEF_NON_ACTIVE'));
            return;
        }

        // Htaccess ?
        if (!is_file(JPATH_SITE.'/.htaccess')) {
            $lang = JFactory::getLanguage();
            $lang->load('plg_system_fmminimize', JPATH_ADMINISTRATOR);
            JError::raiseError(500, JText::_('PLG_SYSTEM_FMMINIMIZE_HTACCESS_NOT_PRESENT'));
            return;
        }

        // Récupération de l'URL
        $u = JURI::getInstance();
        $url = $u->getPath();

        // Test si l'URL est concernée
        if (preg_match('#'.JURI::base(true).'(.*?)\.(mini|limit|crop|deform|fill)\.(\d*)x(\d*)\.jpg#', $url, $out)) {

            // Récupération des paramètres
            $path = $out[1];
            $type = $out[2];
            $w = $out[3];
            $h = $out[4];

            // Configuration plugin
            $cache = $this->params->get('keep_cache', 1);
            $cacheFolder = $this->params->get('cache_folder', 'cache_folder');
            $quality = $this->params->get('quality', 80);

            // Variables
            $scale = 1;
            $fullpath = urldecode(JPATH_BASE.str_replace('%20', ' ', $path));

            // Test fichier
            if (!file_exists($fullpath)) {
                $lang = JFactory::getLanguage();
                $lang->load('plg_system_fmminimize', JPATH_ADMINISTRATOR);
                JError::raiseError(500, sprintf(JText::_('PLG_SYSTEM_FMMINIMIZE_FILE_NOT_EXISTS'), $fullpath));
                exit();
            }

            // Fichier destination
            if ($cacheFolder == 'cache_folder') {
                // Definitions
                $cachePath = JPATH_SITE.'/cache/plg_system_fmminimize';
                $content = '<!DOCTYPE html><title></title>';
                $newpath = $cachePath.$url;
                // Création arborescence
                jimport('joomla.filesystem.file');
                jimport('joomla.filesystem.folder');
                $filename = JFile::getName($url);
                $parts = explode(DIRECTORY_SEPARATOR, substr(str_replace($filename, '', $url), 1, -1));
                $currentPath = array();
                JFile::write($cachePath.'/index.html', $content);
                foreach ($parts as $rep) {
                    array_push($currentPath, $rep);
                    $dir = implode(DIRECTORY_SEPARATOR, $currentPath);
                    if (!JFolder::exists($cachePath.'/'.$dir)) {
                        JFile::write($cachePath.'/'.$dir.'/index.html', $content);
                    }
                }
            }
            else {
                $newpath = $fullpath.'.'.$type.'.'.$w.'x'.$h.'.jpg';
            }

            if(!$cache || !file_exists($newpath)) {

                // 1 - Définition des dimensions finales
                $origsize = getimagesize($fullpath);

                if ((($w == 0 || $h == 0) && $type != 'limit') || ($w == 0 && $h == 0)) {
                    // Dimensions qui ne permettent pas de travailler, on garde les dimensions de base
                    $w = $origsize[0];
                    $h = $origsize[1];
                }

                $rw = min($w/$origsize[0], 1);
                $rh = min($h/$origsize[1], 1);

                if ($type == 'limit' && $w == 0 && $h != 0) {
                    // Echelle à adopter pour un limit où l'on n'a pas définit la largeur
                    $scale = $rh;
                }
                elseif ($type == 'limit' && $h == 0 && $w != 0) {
                    // Echelle à adopter pour un limit où l'on n'a pas définit la hauteur
                    $scale = $rw;
                }
                else {
                    // Echelle à adopter pour les autres cas
                    $scale = ($rw < $rh) ? $rw : $rh;
                    $invertScale = $rh + $rw - $scale;
                    $mw = $origsize[0] * $invertScale;
                    $mh = $origsize[1] * $invertScale;
                }

                $dw = $origsize[0] * $scale;
                $dh = $origsize[1] * $scale;

                // 2 - Détection extension image d'origine et chargement
                $pos = strrpos($path, '.');
                $ext = strtolower(substr($path, $pos+1));
                switch($ext) {
                    case 'png' :
                        $src = imagecreatefrompng($fullpath);
                    break;
                    case 'gif' :
                        $src = imagecreatefromgif($fullpath);
                    break;
                    case 'jpg' :
                    case 'jpeg' :
                        $src = imagecreatefromjpeg($fullpath);
                    break;
                    default :
                        $lang = JFactory::getLanguage();
                        $lang->load('plg_system_fmminimize', JPATH_ADMINISTRATOR);
                        JError::raiseError(sprintf(JText::_('PLG_SYSTEM_FMMINIMIZE_ERROR_EXTENSION'), $fullpath, $ext));
                        exit();
                    break;
                }

                // 3 - Création et Remplissage de l'image
                $new =  imagecreatetruecolor($w, $h);
                switch ($type) {
                    case 'mini':
                        // MINI : créé une image de la taille précisée en gardant l'échelle maximum en longueur ou hauteur
                        // si le format d'origine est différent, une partie de l'image est coupée
                        imagecopyresampled($new, $src, ($w-$mw)/2, ($h-$mh)/2, 0, 0, $mw, $mh, $origsize[0], $origsize[1]);
                        break;
                    case 'crop':
                        // CROP : créé une image de la taille précisée sans redimensionner (on capture une partie de l'image en taille réelle)en completant avec du noir si le format n'est
                        // si la miniature demandée est plus grande que l'image d'origine, on complète avec du noir
                        imagecopyresized($new, $src, ($w-$origsize[0])/2, ($h-$origsize[1])/2, 0, 0, $origsize[0], $origsize[1], $origsize[0], $origsize[1]);
                        break;
                    case 'fill':
                        // FILL : créé une image de la taille précisée en completant avec du noir si le format n'est
                        // pas le même que celui d'origine
                        imagecopyresampled($new, $src, ($w-$dw)/2, ($h-$dh)/2, 0, 0, $dw, $dh, $origsize[0], $origsize[1]);
                        break;
                    case 'deform':
                        // DEFORM : créé une image de la taille précisée en étirant / déformant l'image
                        imagecopyresampled($new, $src, 0, 0, 0, 0, $w, $h, $origsize[0], $origsize[1]);
                        break;
                    default:
                        // LIMIT : conserve le format de l'image d'origine et produit une vignette de la taille maximum
                        // dans les limites données
                        $new =  imagecreatetruecolor($dw, $dh);
                        imagecopyresampled($new, $src, 0, 0, 0, 0, $dw, $dh, $origsize[0], $origsize[1]);
                        break;
                }


                // 4 - Enregistrement cache / Affichage
                if($cache) {
                    imagejpeg($new, $newpath, $quality);
                }
                else {
                    header ('Content-type: image/jpg');
                    imagejpeg($new, null, $quality);
                    exit();
                }

            }

            // Chargement cache
            $data = file_get_contents($newpath);
            // Thanks to Alexander Makarov for his fix
            $LastModified_unix = filemtime($newpath);
            $LastModified = gmdate("D, d M Y H:i:s \G\M\T", $LastModified_unix);
            $IfModifiedSince = false;
            if (isset($_ENV['HTTP_IF_MODIFIED_SINCE']))
                $IfModifiedSince = strtotime(substr($_ENV['HTTP_IF_MODIFIED_SINCE'], 5));
            if (isset($_SERVER['HTTP_IF_MODIFIED_SINCE']))
                $IfModifiedSince = strtotime(substr($_SERVER['HTTP_IF_MODIFIED_SINCE'], 5));
            if ($IfModifiedSince && $IfModifiedSince >= $LastModified_unix) {
                header($_SERVER['SERVER_PROTOCOL'] . ' 304 Not Modified');
                exit;
            }

            header('Last-Modified: '. $LastModified);
            header('Content-type: image/jpg');
            print($data);
            exit();
        }

    }

    /**
     * Traitement des autres opérations
     * -> Redimensionnement auto
     * -> LazyLoad
     */
    public function onBeforeRender()
    {

        $app = JFactory::getApplication();
        $doc = JFactory::getDocument();

        if ($app->isSite()) {

            // Chargement jQuery
            if (JVERSION < 3) {
                if (!$app->get('jquery')) {
                    $app->set('jquery', true);
                    $doc->addScript('https://code.jquery.com/jquery-1.11.1.min.js');
                }
            } else {
                JHtml::_('jquery.framework');
            }

            // Réduction automatique des images
            $script = "";

            if ($this->params->get('redimensionne_auto', 1)) {
                // Paramètres
                $extensions = array();
                if ($this->params->get('redimensionne_auto_jpg')) {
                    array_push($extensions, 'jpg', 'jpeg');
                }
                if ($this->params->get('redimensionne_auto_gif')) {
                    array_push($extensions, 'gif');
                }
                if ($this->params->get('redimensionne_auto_png')) {
                    array_push($extensions, 'png');
                }
                $jquery_selector = trim($this->params->get('jquery_selector'));
                $jquery_selector .= empty($jquery_selector) ? '' : ' ';
                $selector = $jquery_selector.'img[width], '.$jquery_selector.'img[height]';
                // Auto-resize de toutes les images avec attributs width/height
                $script .= "
                jQuery(document).ready(function($) {
                    $.each($('$selector'), function(i, img) {
                        var src = $(img).attr('src');
                        var extension = src.substr((src.lastIndexOf('.')+1)).toLowerCase();
                        if (!src.match(/(.*?)\.(mini|limit|crop|deform|fill)\.(\d+)x(\d+)\.jpg/) && $.inArray(extension, ['".implode("','", $extensions)."']) != -1) {
                            img.src = src+'.deform.'+$(img).attr('width')+'x'+$(img).attr('height')+'.jpg';
                        }
                        ".($this->params->get('lazyload') ? "$(img).addClass('fmminimize_lazy');" : "")."
                    });
                });
                ";
                // Seulement celles dont les dimensions sont différentes à l'affichage
                /*$script .= "
                var fmMinimizeImgArray = [], fmMinimizeAllImages = [];
                jQuery(document).ready(function($) {
                    fmMinimizeAllImages = $('$selector');
                    $.each(fmMinimizeAllImages, function(i, img) {
                        fmMinimizeImgArray.push(img.src);
                        $(img).attr('orig-src', img.src).attr('src', 'data:image/gif;base64,R0lGODlhAQABAAD/ACwAAAAAAQABAAACADs%3D');
                    });
                    $.ajax({
                        url: '".JURI::getInstance()->toString()."?fmmGetDimensions=1',
                        type: 'post',
                        data: 'fmmData[]='+(fmMinimizeImgArray.join('&fmmData[]=')),
                        success: function(a) {
                            var json = $.parseJSON(a);
                            $.each(fmMinimizeAllImages, function(i, img) {
                                var dim = json.dimensions[i].split('x');
                                if (dim[0] && dim[1] && $(img).width() < dim[0] && $(img).height() < dim[1]) {
                                    $(img).attr('src', $(img).attr('src')+'.deform.'+$(img).width()+'x'+$(img).height()+'.jpg');
                                    img.src = $(img).attr('orig-src')+'.deform.'+$(img).width()+'x'+$(img).height()+'.jpg';
                                } else {
                                    img.src = $(img).attr('orig-src');
                                }
                            });
                        }
                    });
                });
                ";*/
            }

            // Lazy Load
            if ($this->params->get('lazyload', 1)) {
                $doc->addScript('https://cdnjs.cloudflare.com/ajax/libs/jquery.lazyload/1.9.1/jquery.lazyload.js');
                $script .= "
                jQuery(document).ready(function($) {
                    $.each($('img.fmminimize_lazy'), function(i, img) {
                        $(img).attr('data-original', img.src).removeAttr('src');
                    });
                    $('img.fmminimize_lazy').lazyload({
                        effect: 'fadeIn'
                    });
                });
                ";
            }


            // Ajout du script
            if (!empty($script)) {
                //$script = str_replace(PHP_EOL, '', $script);
                //$script = preg_replace('/\s+/', ' ', $script);
                $doc->addScriptDeclaration($script);
            }

        }

    }


    /**
     * Ajoute la configuration nécessaire
     * dans le fichier SEF actif
     * TODO Adapter au fichier web.config
     */
    private function writeConfig()
    {

        // Fichier .htaccess
        if (is_file(JPATH_BASE.'/.htaccess')) {
            $content = file_get_contents(JPATH_BASE.'/.htaccess');
            if (!preg_match('/.*Plugin Minimize SEF Section.*/', $content) or !preg_match('/.*\(mini\|limit\|crop\|deform\|fill\).*/', $content)) {
                // Enlever ancienne version
                $content = preg_replace('/(.*)## Begin - Plugin Minimize SEF Section\.([\s\S]*)## End - Plugin Minimize SEF Section\.(.*)/', '', $content);
                $config = '## Begin - Plugin Minimize SEF Section.'
                        ."\r\n".'# Route matching requests to the index.php script'
                        ."\r\n".'RewriteRule   ^(.*?)\.(mini|limit|crop|deform|fill)\.(\d*)x(\d*)\.jpg$ index.php  [L]'
                        ."\r\n".'## End - Plugin Minimize SEF Section.';
                file_put_contents(JPATH_BASE.'/.htaccess', $content."\r\n\r\n".$config);
            }
        }

    }

}